// Demo 18: MoveableSquare.java
//
//   A Square with Movement information added.
//       A Simple Example of Subclassing.
//
//          Another Animation Method
//
//       by Charles W. Neville
//   Copyright Charles W. Neville, June 1999
//

// There is another animation method which is conceptually
// less interesting, but usually more efficient on a single
// processor machine.  Instead of having many separate 
// animation Threads running, one for each moving object, have
// just one animation Thread that moves and redraws all the
// objects in a classical "event loop".  In this approach,
// moving objects contain all the information and methods needed
// to move and draw themselves, just as Square objects do, plus
// information about where they should move next and, possibly,
// how far they should move.  MoveableSquare.java and the next
// few demos illustrate this approach.

//  A MoveableSquare object should know
//    1. Everything a Square object knows.
//    2. A direction vector so it knows in which direction
//       it is heading (state variables).
//    3. A distance scalar so it knows how far it is to move
//       in the given direction.
//    4. How to set and report its heading (methods).
//    5. How to set and report its distance (methods).

import java.awt.*;

public class MoveableSquare extends Square {

  // all the Square object state variables are
  // automatically inherited.  No need to repeat
  // ourselves.

  // all the Square methods are inhereted
  // too, so we don't need to repeat them.

  // Additional object state variables and methods
  // unique to this class:

  int rx, ry;  // x and y coordinates of the heading vector.
  int d;       // distance scalar.
  int dt;      // distance traveled so far scalar.

  // A Square Constructor -- Like most constructors, this one
  //    simply initializes our object state variables.
  public MoveableSquare(int x, int y, int size,
                int rx, int ry, int d, Color color) {

     super(x, y, size, color);  // a call to super() invokes the
                                // the parent class constructor,
                                // so we don't have to repeat
                                // ourselves

     // now deal with the stuff unique to MoveableSquares
     this.rx = rx;
     this.ry = ry;
     this.d = d;

     this.dt = 0;

  } // Constructor MoveableSquare()

  // set our rx heading coordinate
  public void setRX(int rx) {

     this.rx = rx;

  } // setRX()

  // return our rx heading coordinate
  public int getRX() {

     return rx;

  } // getRX()

  // set our ry heading coordinate
  public void setRY(int ry) {

     this.ry = ry;

  } // setRY()

  // return our ry heading coordinate
  public int getRY() {

     return ry;

  } // getRY()

  // set our distance
  public void setD(int d) {

     this.d = d;

  } // setD()

  // return our distance
  public int getD() {

     return d;

  } // getD()

   // set our distance traveled
  public void setDT(int dt) {

     this.dt = dt;

  } // setDT()

  // return our distance traveled
  public int getDT() {

     return dt;

  } // getDT()

  // increment our distance traveled by x
  public void incrDT(int x) {

     dt = dt + x;

  } // incrDT()
   
} // class MoveableSquare